@@ -18,7 +18,7 @@
 from b2sdk.api import B2Api
 from b2sdk.bucket import Bucket
 from b2sdk.bucket import BucketFactory
-from b2sdk.raw_api import ALL_CAPABILITIES
+from b2sdk.raw_api import ALL_CAPABILITIES, REALM_URLS
 
 # encryption
 
@@ -34,7 +34,7 @@ from b2sdk.encryption.types import EncryptionMode
 from b2sdk.account_info.abstract import AbstractAccountInfo
 from b2sdk.account_info.in_memory import InMemoryAccountInfo
 from b2sdk.account_info.sqlite_account_info import SqliteAccountInfo
-from b2sdk.account_info.sqlite_account_info import B2_ACCOUNT_INFO_ENV_VAR, B2_ACCOUNT_INFO_DEFAULT_FILE
+from b2sdk.account_info.sqlite_account_info import B2_ACCOUNT_INFO_ENV_VAR, B2_ACCOUNT_INFO_DEFAULT_FILE, XDG_CONFIG_HOME_ENV_VAR
 from b2sdk.account_info.stub import StubAccountInfo
 from b2sdk.account_info.upload_url_pool import UploadUrlPool
 from b2sdk.account_info.upload_url_pool import UrlPoolAccountInfo
@@ -64,9 +64,24 @@ from b2sdk.utils import trace_call
 
 from b2sdk.file_version import FileIdAndName
 from b2sdk.file_version import FileVersionInfo
+from b2sdk.file_version import FileVersionInfoFactory
 from b2sdk.large_file.part import Part
 from b2sdk.large_file.unfinished_large_file import UnfinishedLargeFile
 
+# file lock
+
+from b2sdk.file_lock import BucketRetentionSetting
+from b2sdk.file_lock import FileLockConfiguration
+from b2sdk.file_lock import FileRetentionSetting
+from b2sdk.file_lock import LegalHold
+from b2sdk.file_lock import NO_RETENTION_BUCKET_SETTING
+from b2sdk.file_lock import NO_RETENTION_FILE_SETTING
+from b2sdk.file_lock import RetentionMode
+from b2sdk.file_lock import RetentionPeriod
+from b2sdk.file_lock import UNKNOWN_BUCKET_RETENTION
+from b2sdk.file_lock import UNKNOWN_FILE_LOCK_CONFIGURATION
+from b2sdk.file_lock import UNKNOWN_FILE_RETENTION_SETTING
+
 # progress reporting
 
 from b2sdk.progress import AbstractProgressListener
@@ -172,6 +187,7 @@ from b2sdk.sync.scan_policies import DEFAULT_SCAN_MANAGER
 from b2sdk.sync.scan_policies import IntegerRange
 from b2sdk.sync.scan_policies import RegexSet
 from b2sdk.sync.scan_policies import ScanPoliciesManager
+from b2sdk.sync.scan_policies import convert_dir_regex_to_dir_prefix_regex
 from b2sdk.sync.sync import KeepOrDeleteMode
 from b2sdk.sync.sync import Synchronizer
 from b2sdk.sync.sync import zip_folders
