@@ -11,6 +11,7 @@
 from b2sdk.account_info.exception import AccountInfoError
 from b2sdk.account_info.exception import CorruptAccountInfo
 from b2sdk.account_info.exception import MissingAccountData
+from b2sdk.exception import AccessDenied
 from b2sdk.exception import AlreadyFailed
 from b2sdk.exception import B2ConnectionError
 from b2sdk.exception import B2Error
@@ -30,7 +31,6 @@ from b2sdk.exception import CapabilityNotAllowed
 from b2sdk.exception import CapExceeded
 from b2sdk.exception import ChecksumMismatch
 from b2sdk.exception import ClockSkew
-from b2sdk.exception import CommandError
 from b2sdk.exception import Conflict
 from b2sdk.exception import ConnectionReset
 from b2sdk.exception import DestFileNewer
@@ -50,6 +50,7 @@ from b2sdk.exception import NonExistentBucket
 from b2sdk.exception import NotAllowedByAppKeyError
 from b2sdk.exception import PartSha1Mismatch
 from b2sdk.exception import RestrictedBucket
+from b2sdk.exception import RetentionWriteError
 from b2sdk.exception import ServiceError
 from b2sdk.exception import StorageCapExceeded
 from b2sdk.exception import TooManyRequests
@@ -67,12 +68,17 @@ from b2sdk.exception import SSECKeyIdMismatchInCopy
 from b2sdk.exception import SSECKeyError
 from b2sdk.exception import WrongEncryptionModeForBucketDefault
 from b2sdk.exception import interpret_b2_error
+from b2sdk.sync.exception import EmptyDirectory
 from b2sdk.sync.exception import EnvironmentEncodingError
 from b2sdk.sync.exception import IncompleteSync
 from b2sdk.sync.exception import InvalidArgument
+from b2sdk.sync.exception import NotADirectory
+from b2sdk.sync.exception import UnableToCreateDirectory
 from b2sdk.sync.exception import UnSyncableFilename
+from b2sdk.sync.exception import check_invalid_argument
 
 __all__ = (
+    'AccessDenied',
     'AccountInfoError',
     'AlreadyFailed',
     'B2ConnectionError',
@@ -93,12 +99,12 @@ __all__ = (
     'CapExceeded',
     'ChecksumMismatch',
     'ClockSkew',
-    'CommandError',
     'Conflict',
     'ConnectionReset',
     'CorruptAccountInfo',
     'DestFileNewer',
     'DuplicateBucketName',
+    'EmptyDirectory',
     'EnvironmentEncodingError',
     'FileAlreadyHidden',
     'FileNameNotAllowed',
@@ -115,9 +121,11 @@ __all__ = (
     'MissingAccountData',
     'MissingPart',
     'NonExistentBucket',
+    'NotADirectory',
     'NotAllowedByAppKeyError',
     'PartSha1Mismatch',
     'RestrictedBucket',
+    'RetentionWriteError',
     'ServiceError',
     'StorageCapExceeded',
     'TooManyRequests',
@@ -129,10 +137,12 @@ __all__ = (
     'UnknownError',
     'UnknownHost',
     'UnrecognizedBucketType',
+    'UnableToCreateDirectory',
     'UnSyncableFilename',
     'UnsatisfiableRange',
     'UnusableFileName',
     'interpret_b2_error',
+    'check_invalid_argument',
     'SSECKeyIdMismatchInCopy',
     'SSECKeyError',
     'WrongEncryptionModeForBucketDefault',
