@@ -19,7 +19,8 @@ from b2sdk.b2http import B2Http
 from b2sdk.cache import AuthInfoCache, DummyCache
 from b2sdk.encryption.setting import EncryptionSetting
 from b2sdk.exception import (InvalidAuthToken, Unauthorized)
-from b2sdk.raw_api import ALL_CAPABILITIES, B2RawApi
+from b2sdk.file_lock import BucketRetentionSetting, FileRetentionSetting, LegalHold
+from b2sdk.raw_api import ALL_CAPABILITIES, B2RawApi, REALM_URLS
 
 logger = logging.getLogger(__name__)
 
@@ -37,6 +38,7 @@ class B2Session(object):
         A facade that supplies the correct api_url and account_auth_token
         to methods of underlying raw_api and reauthorizes if necessary.
     """
+    SQLITE_ACCOUNT_INFO_CLASS = staticmethod(SqliteAccountInfo)
 
     def __init__(self, account_info=None, cache=None, raw_api=None):
         """
@@ -65,7 +67,7 @@ class B2Session(object):
 
         self.raw_api = raw_api or B2RawApi(B2Http())
         if account_info is None:
-            account_info = SqliteAccountInfo()
+            account_info = self.SQLITE_ACCOUNT_INFO_CLASS()
             if cache is None:
                 cache = AuthInfoCache(account_info)
         if cache is None:
@@ -104,7 +106,7 @@ class B2Session(object):
         :param str application_key: user's :term:`application key`
         """
         # Authorize
-        realm_url = self.account_info.REALM_URLS.get(realm, realm)
+        realm_url = REALM_URLS.get(realm, realm)
         response = self.raw_api.authorize_account(realm_url, application_key_id, application_key)
         account_id = response['accountId']
         allowed = response['allowed']
@@ -119,7 +121,8 @@ class B2Session(object):
             auth_token=response['authorizationToken'],
             api_url=response['apiUrl'],
             download_url=response['downloadUrl'],
-            minimum_part_size=response['recommendedPartSize'],
+            absolute_minimum_part_size=response['absoluteMinimumPartSize'],
+            recommended_part_size=response['recommendedPartSize'],
             application_key=application_key,
             realm=realm,
             s3_api_url=response['s3ApiUrl'],
@@ -139,6 +142,7 @@ class B2Session(object):
         cors_rules=None,
         lifecycle_rules=None,
         default_server_side_encryption=None,
+        is_file_lock_enabled: Optional[bool] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.create_bucket,
@@ -149,6 +153,7 @@ class B2Session(object):
             cors_rules=cors_rules,
             lifecycle_rules=lifecycle_rules,
             default_server_side_encryption=default_server_side_encryption,
+            is_file_lock_enabled=is_file_lock_enabled,
         )
 
     def create_key(
@@ -283,6 +288,8 @@ class B2Session(object):
         content_type,
         file_info,
         server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.start_large_file,
@@ -291,6 +298,8 @@ class B2Session(object):
             content_type,
             file_info,
             server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
     def update_bucket(
@@ -303,6 +312,7 @@ class B2Session(object):
         lifecycle_rules=None,
         if_revision_is=None,
         default_server_side_encryption: Optional[EncryptionSetting] = None,
+        default_retention: Optional[BucketRetentionSetting] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.update_bucket,
@@ -314,6 +324,7 @@ class B2Session(object):
             lifecycle_rules=lifecycle_rules,
             if_revision_is=if_revision_is,
             default_server_side_encryption=default_server_side_encryption,
+            default_retention=default_retention,
         )
 
     def upload_file(
@@ -326,6 +337,8 @@ class B2Session(object):
         file_infos,
         data_stream,
         server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         return self._wrap_token(
             self.raw_api.upload_file,
@@ -338,6 +351,8 @@ class B2Session(object):
             file_infos,
             data_stream,
             server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
     def upload_part(
@@ -379,6 +394,8 @@ class B2Session(object):
         destination_bucket_id=None,
         destination_server_side_encryption: Optional[EncryptionSetting] = None,
         source_server_side_encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         return self._wrap_default_token(
             self.raw_api.copy_file,
@@ -391,6 +408,8 @@ class B2Session(object):
             destination_bucket_id=destination_bucket_id,
             destination_server_side_encryption=destination_server_side_encryption,
             source_server_side_encryption=source_server_side_encryption,
+            file_retention=file_retention,
+            legal_hold=legal_hold,
         )
 
     def copy_part(
@@ -512,3 +531,31 @@ class B2Session(object):
         response = f(upload_url, upload_auth_token, *args, **kwargs)
         self.account_info.put_large_file_upload_url(file_id, upload_url, upload_auth_token)
         return response
+
+    def update_file_retention(
+        self,
+        file_id,
+        file_name,
+        file_retention: FileRetentionSetting,
+        bypass_governance: bool = False,
+    ):
+        return self._wrap_default_token(
+            self.raw_api.update_file_retention,
+            file_id,
+            file_name,
+            file_retention,
+            bypass_governance,
+        )
+
+    def update_file_legal_hold(
+        self,
+        file_id,
+        file_name,
+        legal_hold: LegalHold,
+    ):
+        return self._wrap_default_token(
+            self.raw_api.update_file_legal_hold,
+            file_id,
+            file_name,
+            legal_hold,
+        )
