@@ -11,12 +11,15 @@
 import logging
 import concurrent.futures as futures
 
+from typing import Optional
+
 from b2sdk.encryption.setting import EncryptionMode, EncryptionSetting
 from b2sdk.exception import (
     AlreadyFailed,
     B2Error,
     MaxRetriesExceeded,
 )
+from b2sdk.file_lock import FileRetentionSetting, LegalHold
 from b2sdk.file_version import FileVersionInfoFactory
 from b2sdk.stream.progress import ReadingStreamWithProgress
 from b2sdk.stream.hashing import StreamWithHash
@@ -78,7 +81,9 @@ class UploadManager(metaclass=B2TraceMetaAbstract):
         content_type,
         file_info,
         progress_listener,
-        encryption: EncryptionSetting = None,
+        encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         f = self.get_thread_pool().submit(
             self._upload_small_file,
@@ -89,6 +94,8 @@ class UploadManager(metaclass=B2TraceMetaAbstract):
             file_info,
             progress_listener,
             encryption,
+            file_retention,
+            legal_hold,
         )
         return f
 
@@ -203,7 +210,9 @@ class UploadManager(metaclass=B2TraceMetaAbstract):
         content_type,
         file_info,
         progress_listener,
-        encryption: EncryptionSetting,
+        encryption: Optional[EncryptionSetting] = None,
+        file_retention: Optional[FileRetentionSetting] = None,
+        legal_hold: Optional[LegalHold] = None,
     ):
         content_length = upload_source.get_content_length()
         exception_info_list = []
@@ -232,6 +241,8 @@ class UploadManager(metaclass=B2TraceMetaAbstract):
                             file_info,
                             input_stream,
                             server_side_encryption=encryption,  # todo: client side encryption
+                            file_retention=file_retention,
+                            legal_hold=legal_hold,
                         )
                         if content_sha1 == HEX_DIGITS_AT_END:
                             content_sha1 = input_stream.hash
